/***************************************************************************
 *   Copyright (C) 2009 by Matthew Dellinger   *
 *   mdelling@vt.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "rt_task.h"
#include <chronos/chronos.h>

#define tsprint(x) x.tv_sec << " sec + " << x.tv_nsec << " nsec"
#define tsprintp(x) x->tv_sec << " sec + " << x->tv_nsec << " nsec"
#define THREAD_OUTPUT if(me->verbose() && accrue) *me->output() << me->thread_id()
#define THREAD_OUTPUT_F if(me->verbose()) *me->output() << me->thread_id()

RtTester *tester;

void Task::CpuUsage(RtTester *t){
 cpu_usage_task=t->usage_per();
 high_freq= t->CalcHighFreq();
 n_freq=0;// t->CalcSlowdownFactors();
 cpu_usage_end=t->usage_exec_time();

}

void Task::SetupTask(RtTester *t, ostringstream *output, chronos_aborts_t *a, double slope) {
	tester = t;
	output_ = output;
	exec_slope_ = slope;
	aborts_ = a;
}
 
/* Hook in here for different kinds of workloads */
void Task::Work(long long time_us) {
	burn_cpu_abort(time_us, exec_slope_, abort_flag_);
}

int Task::start(pthread_attr_t *attr) {
	return pthread_create(&thread, attr, task, (void *)this);
}

void Task::BeginRtSegment(int mutex_flag,  chronos_mutex_t *m0 , chronos_mutex_t *m1) {
    mutex_flag=(10*(cpu_usage_end)*100*cpu_usage_task)/10000;
      // m=NULL;
    //     cout << " mutex_flag " << mutex_flag << endl;
	begin_rtseg_self(TASK_RUN_PRIO, utility(),cpu_usage_task, mutex_flag, high_freq,n_freq, m0, m1,locked_usage(), &deadline_, period_ts(),
		unlocked_usage() + locked_usage());

}

void Task::SetupAbortHandler() {
	if(tester->use_hua() && abort_utility() > 0)
		add_abort_handler_selfnodeadline(abort_utility(),
				tester->lock_time()*tester->lock_count());
}

void Task::EndRtSegment() {
	end_rtseg_self(TASK_CLEANUP_PRIO);
}

long Task::timespec_subtract(struct timespec *x, struct timespec *y) {
	long sec, nsec;
	sec = x->tv_sec - y->tv_sec;
	nsec = x->tv_nsec - y->tv_nsec;
	if(nsec < 0) {
		nsec += BILLION;
		sec--;
	}

	return sec * MILLION + nsec/THOUSAND;
}

void Task::UpdateDeadline() {
	unsigned long carry;
	unsigned long long nsec, offset = period_ * releases_;

	nsec = start_time_->tv_nsec + offset * THOUSAND;
	carry = nsec/BILLION;

	deadline_.tv_nsec = nsec % BILLION;
	deadline_.tv_sec = start_time_->tv_sec + carry;
}

void *Task::task(void *value) {
	Task *me = (Task *)value;
	unsigned long cpus = me->cpus();
	bool accrue = true;
	long tardiness;
	struct timespec end_time;
	struct sched_param param;   
        chronos_mutex_t *m0=NULL;
        chronos_mutex_t *m1=NULL;
 

	me->set_thread_id(gettid());
	me->set_abort_flag();
	sched_setaffinity(0, sizeof(long), (cpu_set_t *) &cpus);
	tester->mark_started();
	param.sched_priority = TASK_START_PRIO;
	pthread_setschedparam(pthread_self(), SCHED_FIFO, &param);
        int mutex_flag=0;
        int l;
    
	while(tester->barrier()) { }
          
	while(true) {

		accrue = me->AddRelease();
		me->UpdateDeadline();
             
       //         mutex_flag=0;
		THREAD_OUTPUT << "/" << me->task_id() << " - Deadline: " << tsprintp(me->deadline()) << endl;

		/* Declare ourself as a real time task */
              me->CpuUsage(tester);
		if(tester->chronos()) {
			/*me->SetupAbortHandler();
                   //    if(chronos_mutex_owner(tester->lock(0))!=0)mutex_flag=1;
                         THREAD_OUTPUT << "in task bef begin " << endl;
                          m=tester->lock(0);
                       THREAD_OUTPUT << "tester->lock(0) " << tester->lock(0) <<endl;
			me->BeginRtSegment(mutex_flag,m);
                      THREAD_OUTPUT << "in task after begin " << endl;
               */
                        me->SetupAbortHandler();   
                   //    if(chronos_mutex_owner(tester->lock(0))!=0)mutex_flag=1;
                         THREAD_OUTPUT << "in task bef begin " << endl;
               if(me->locking()){

   
            
                       l = me->lock(0);
                          m0=tester->lock(l);
                    //   THREAD_OUTPUT << " tester->lock(0)_sonal3 " << tester->lock(l) << " l "  << l << endl;
                       l = me->lock(1);
                          m1=tester->lock(l);
                    //  THREAD_OUTPUT << " tester->lock(1)_sonal3 " << tester->lock(l) << " l " << l <<  endl;
                     
                  //   l = me->lock(0);
                       //  m0=tester->lock(0);
                     //  THREAD_OUTPUT << " tester->lock(0)_sonal3 " << tester->lock(0) << endl;
                    //   l = me->lock(1);
                      //m1=tester->lock(l);
                   //   THREAD_OUTPUT << " tester->lock(1)_sonal3 " << tester->lock(l) << " l " << l <<  endl;
			}
                         me->BeginRtSegment(mutex_flag,m0,m1);
                      THREAD_OUTPUT << "in task after begin " << endl;


		} else {
			struct sched_param param;
			param.sched_priority = TASK_RUN_PRIO;
			sched_setscheduler(0, SCHED_FIFO, &param);
		}

		if(me->locking() && me->nested()) {
			int last;
			for(int lock_num = 0; lock_num < me->CountLocks(); lock_num++) {
				int r = me->lock(lock_num);
				THREAD_OUTPUT << " locking " << r << endl;
				if(chronos_mutex_lock(tester->lock(r)) == -1)
					break; THREAD_OUTPUT << "in task bef begin " << endl;
				else
					last = lock_num;
				THREAD_OUTPUT << " locked " << r << endl;
			}

			THREAD_OUTPUT << " burning - locked" << endl;
			me->Work(me->locked_usage());

			for(int lock_num = last; lock_num >= 0; lock_num--) {
				int r = me->lock(lock_num);
				THREAD_OUTPUT << " unlocking " << r << endl;
				chronos_mutex_unlock(tester->lock(r));
			}
		} else if(me->locking()){
			for(int lock_num = 0; lock_num < me->CountLocks(); lock_num++) {
				int r = me->lock(lock_num);
				THREAD_OUTPUT << " locking " << r << endl;
				if(chronos_mutex_lock(tester->lock(r)) == -1)
					break;
				THREAD_OUTPUT << " locked " << r << endl;
				THREAD_OUTPUT << " burning - locked" << endl;
                           //    THREAD_OUTPUT << " chronos_mutex_owner(tester->lock(r)) " << chronos_mutex_owner(tester->lock(r)) << endl;  
                            //   THREAD_OUTPUT << " chronos_mutex_owner(tester->lock(1)) " << chronos_mutex_owner(tester->lock(1)) << endl;  
                           //   THREAD_OUTPUT << " (tester->lock(r)) " << (tester->lock(r)) << " r " << r << endl; 
                            //   THREAD_OUTPUT << " (tester->lock(0)) " << (tester->lock(0)) << endl; 
				me->Work(me->locked_usage()/me->CountLocks());
				THREAD_OUTPUT << " unlocking " << r << endl;
				chronos_mutex_unlock(tester->lock(r));
			}
		} 
                
		THREAD_OUTPUT << " burning - unlocked" << endl;

		me->Work(tester->usage_exec_time()*me->unlocked_usage()/10);
               THREAD_OUTPUT << " tester->usage_exec_time() " << tester->usage_exec_time() << endl;
		clock_gettime(CLOCK_REALTIME, &end_time);

		if(tester->chronos())
			me->EndRtSegment();

		tardiness = timespec_subtract(me->deadline(), &end_time);
//sonal3
		if(!accrue)
			break;
		else if(me->aborted())
			me->AddAborted();
		else if(tardiness >= 0) {
			me->AddDeadline();
			me->AddUtility();

			THREAD_OUTPUT_F << "/" << me->task_id() << " end time: " << tsprint(end_time)
			<< " - Deadline " << me->releases()
			<< ": " << tsprintp(me->deadline()) << endl;

			clock_gettime(CLOCK_REALTIME, &end_time);
			tardiness = timespec_subtract(me->deadline(), &end_time);
			if(tardiness > 0)
				usleep(tardiness);
		}
		else {
			me->set_max_tardiness(tardiness);
			if(me->verbose())
				THREAD_OUTPUT_F << "/" << me->task_id() << " task failed:"
				<< " end time: " << tsprint(end_time) << " - Deadline "
				<< me->releases() << ": " << tsprintp(me->deadline()) << endl;
		}
	}

	pthread_exit(NULL);
}
