#include "test.h"

unsigned long mask = 1;
chronos_mutex_t *r1;

struct timespec period;

void *task1(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask), (cpu_set_t *) &mask);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	period.tv_sec = 2;
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	test_val *= 7;

	//actual task block
	THREAD_OUTPUT("thread %d - locking lock 1\n", thread_id);
	chronos_mutex_lock(r1);
	THREAD_OUTPUT("thread %d - locked lock 1\n", thread_id);
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	test_val -= 9;
	burn_cpu(2000000, exec_slope);
	THREAD_OUTPUT("thread %d - unlocking lock 1\n", thread_id);
	test_val -= 13;
	chronos_mutex_unlock(r1);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	pthread_exit(NULL);
}

void *task2(void *p)
{
	int thread_id = gettid(), result;
	sched_setaffinity(0, sizeof(mask), (cpu_set_t *) &mask);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	period.tv_sec = 1;
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	test_val *= 11;

	//actual task block
	THREAD_OUTPUT("thread %d - locking lock 1\n", thread_id);
	result = chronos_mutex_lock(r1);
	if(result == 0)
		test_val *= 15;
	THREAD_OUTPUT("thread %d - locked lock 1\n", thread_id);
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	THREAD_OUTPUT("thread %d - unlocking lock 1\n", thread_id);
	chronos_mutex_unlock(r1);
	test_val -= 17;
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	pthread_exit(NULL);
}

int main(int argc, char* argv[])
{
	pthread_t t1, t2;
	sched = SCHED_RT_RMA | SCHED_FLAG_PI;

	r1 = new chronos_mutex_t();
	chronos_mutex_init(r1);

	while ((c = getopt(argc, argv, optstring)) != -1)
	{
		switch (c) {
		case 'v' :
			verbose = true;
			break;
		case 'h' :
			cout << "A simple test for priority inheritance behavior under single-core scheduling. Use -v for detailed output" << endl;
			return 0;
		}
	}

	exec_slope = get_slope();
	if(exec_slope == 0) {
		cout << "Error: No slope found" << endl;
		return 0;
	}

	period.tv_sec = 0;
	period.tv_nsec = 0;

	if(prep_test())
		return 1;

	if(set_scheduler(sched, -1, mask)) {
		cout << "Error: Unable to set scheduler" << endl;
		cout << "Are you running as root?" << endl;
	}

	test_val +=5;

	pthread_create(&t1, NULL, task1, NULL);
	sleep(1);
	pthread_create(&t2, NULL, task2, NULL);
	sleep(1);
	pthread_join(t2, NULL);
	pthread_join(t1, NULL);

	if(test_val == 7543)
		cout << "Single-core PI test successful" << endl;
	else
		cout << "Single-core PI test failed: " << test_val << endl;

	chronos_mutex_destroy(r1);
	delete r1;
	return 0;
}

