#include "test.h"

unsigned long mask1 = 1, mask2 = 2, mask3 = 3;
struct timespec period;

/* Starts on CPU 0, runs on CPU 0 */
void *task1(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask1), (cpu_set_t *) &mask1);
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	test_val *= 7; //56, 0.0 seconds
	period.tv_sec = 4;
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	//actual task block - 3.0 seconds
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(200000, exec_slope);
	test_val *= 11; //517, 0.2 seconds
	burn_cpu(1800000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	test_val *= 19; //143317 2.0 seconds
	pthread_exit(NULL);
}

/* Starts on CPU 0, runs on CPU 1 */
void *task2(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask1), (cpu_set_t *) &mask1);
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	test_val -= 9; //47, 0.0 seconds
	period.tv_sec = 1;
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	//actual task block - 2.0 seconds
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(400000, exec_slope);
	test_val -= 13; //504, 0.4 seconds
	burn_cpu(2600000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	test_val *= 23; //3295808, 3.0 seconds
	pthread_exit(NULL);
}

/* Starts on CPU 1, runs on CPU 0 */
void *task3(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask2), (cpu_set_t *) &mask2);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	test_val *= 15; //7560, 1.0 seconds
	period.tv_sec = 3;
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	//actual task block - 2.0 seconds
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(200000, exec_slope);
	test_val -= 21; //143296 2.2 seconds
	burn_cpu(1800000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	test_val *= 27; //88986141, 4.0 seconds
	pthread_exit(NULL);
}

/* Starts on CPU 1, runs on CPU 1 */
void *task4(void *p)
{
	int thread_id = gettid();
	sched_setaffinity(0, sizeof(mask2), (cpu_set_t *) &mask2);
	sched_setaffinity(0, sizeof(mask1), (cpu_set_t *) &mask1);
	sched_setaffinity(0, sizeof(mask3), (cpu_set_t *) &mask3);

	THREAD_OUTPUT("thread %d - start\n", thread_id);
	test_val -= 17; //7543, 1.0 seconds
	period.tv_sec = 2;
	begin_rtseg_selfbasic(RUN_PRIO, NULL, &period);

	//actual task block - 2.0 seconds
	THREAD_OUTPUT("thread %d - burning cpu\n", thread_id);
	burn_cpu(300000, exec_slope);
	test_val -= 25; //3295783, 3.3 seconds
	burn_cpu(1700000, exec_slope);
	end_rtseg_self(END_PRIO);
	THREAD_OUTPUT("thread %d - end\n", thread_id);
	test_val -= 29; //88986112
	pthread_exit(NULL);
}

int main(int argc, char* argv[])
{
	pthread_t t1, t2, t3, t4;
	int proc_count = 1;
	sched = SCHED_RT_GFIFO;

	while ((c = getopt (argc, argv, optstring)) != -1)
	{
		switch (c) {
		case 'v' :
			verbose = true;
			break;
		case 'h' :
			cout << "A simple test for proper apc global scheduling behavior. Use -v for detailed output" << endl;
			return 0;
		}
	}

#ifdef _SC_NPROCESSORS_ONLN
	proc_count = sysconf(_SC_NPROCESSORS_ONLN);
#endif

	if(proc_count == 1) {
		cout << "Global scheduling test not applicable to single-core machine" << endl;
		return 0;
	}

	exec_slope = get_slope();
	if(exec_slope == 0) {
		cout << "Error: No slope found" << endl;
		return 0;
	}

	period.tv_sec = 0;
	period.tv_nsec = 0;

	if(prep_test())
		return 1;

	if(set_scheduler(sched, RUN_PRIO, mask3)) {
		cout << "Error: Unable to set scheduler" << endl;
		cout << "Are you running as root?" << endl;
	}

	test_val +=5;

	pthread_create(&t1, NULL, task1, NULL);
	pthread_create(&t2, NULL, task2, NULL);
	sleep(1);
	pthread_create(&t3, NULL, task3, NULL);
	pthread_create(&t4, NULL, task4, NULL);
	sleep(1);
	pthread_join(t2, NULL);
	pthread_join(t1, NULL);
	pthread_join(t3, NULL);
	pthread_join(t4, NULL);

	if(test_val == 88986112)
		cout << "Global scheduling test (1/2) successful" << endl;
	else
		cout << "Global scheduling test (1/2) failed: " << test_val << endl;

	return 0;
}




