/***************************************************************************
 *   Copyright (C) 2009-2011 Virginia Tech Real-Time Systems Lab	   *
 *   Written by Matthew Dellinger					   *
 *   mdelling@vt.edu							   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or	   *
 *   (at your option) any later version.				   *
 *									   *
 *   This program is distributed in the hope that it will be useful,	   *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of	   *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the	   *
 *   GNU General Public License for more details.			   *
 *									   *
 *   You should have received a copy of the GNU General Public License	   *
 *   along with this program; if not, write to the			   *
 *   Free Software Foundation, Inc.,					   *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.		   *
 ***************************************************************************/

#ifndef CHRONOS_H
#define CHRONOS_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sched.h>
#include <time.h>
#include <unistd.h>
#include <sys/syscall.h>
#include <sys/types.h>

/* System call numbers for 2.6.33.7-chronos */
#ifdef __i386__
#define __NR_do_rt_seg			338
#define __NR_do_chronos_mutex		339
#define __NR_set_scheduler		340
#else
#define __NR_do_rt_seg			300
#define __NR_do_chronos_mutex		301
#define __NR_set_scheduler		302
#endif

/* Masks for getting the scheduler from userspace
 * 1 bit - global
 * 7 bits - scheduler number
 * 8 bits - flags
 */
#define SCHED_GLOBAL_MASK		0x8000
#define SCHED_NUM_MASK			0x7F00
#define SCHED_FLAGS_MASK		0x00FF
/* Scheduler - behaviors for each flag may or may not be defined */
#define SCHED_RT_FIFO 			0x0000
#define SCHED_RT_RMA 			0x0100
#define SCHED_RT_EDF 			0x0200
#define SCHED_RT_HVDF			0x0300
#define SCHED_RT_LBESA			0x0400
#define SCHED_RT_DASA_ND		0x0500
#define SCHED_RT_DASA			0x0600
#define SCHED_RT_BASE_EDF               0x0800
#define SCHED_RT_STATIC_EDF             0x0900
#define SCHED_RT_CC_EDF                 0x0A00
#define SCHED_RT_LAEDF                  0x0B00
#define SCHED_RT_SNOWDON_MIN            0x0C00
#define SCHED_RT_DRA                    0x0D00
#define SCHED_RT_DRA_OTE                0x0E00
#define SCHED_RT_AGR1                   0x0F00
#define SCHED_RT_AGR2                   0x1000
#define SCHED_RT_HS                     0x1100
#define SCHED_RT_DS                     0x1200
#define SCHED_RT_USFI_EDF               0x1300
#define SCHED_RT_REUA                   0x1400
#define SCHED_RT_GFIFO			0x8000
#define SCHED_RT_GRMA			0x8100
#define SCHED_RT_GEDF			0x8200
#define SCHED_RT_GNP_EDF		0x8300
#define SCHED_RT_GHVDF			0x8400
#define SCHED_RT_GMUA			0x8500
#define SCHED_RT_GGUA			0x8600
#define SCHED_RT_NGGUA			0x8700
#define SCHED_RT_GNP_HVDF		0x8800

/* Scheduling Flags */
/* PI == Priority Inheritance
 * HUA == HUA style abort handlers
 * ABORT_FIRST == tasks will be aborted as soon as they fail. 
 * 	Otherwise, the behavior is left unspecified.
 * NO_DEADLOCKS == with deadlock prevention
 *
 * Also, 0x*0 is left open for future flags.
 */
#define SCHED_FLAG_NO_FLAGS		0x00
#define SCHED_FLAG_HUA			0x01
#define SCHED_FLAG_PI			0x02
#define SCHED_FLAG_NO_DEADLOCKS		0x04

/* Operations for multiplexed system calls */
#define RT_SEG_BEGIN			0
#define RT_SEG_END			1
#define RT_SEG_ADD_ABORT		2

#define CHRONOS_MUTEX_REQUEST		0
#define CHRONOS_MUTEX_RELEASE		1
#define CHRONOS_MUTEX_INIT		2
#define CHRONOS_MUTEX_DESTROY		3

/* An actual mutex */
struct mutex_data {
	u_int32_t value;
	int owner;
	unsigned long id;
};

struct rt_data {
//int mutex_flag, unsigned long locked_usage, int high_freq
	int tid;
	int prio;
	unsigned long exec_time;
        int util_taskset;
        unsigned long locked_usage;
        int mutex_flag;
        int high_freq;
	int max_util;
	struct timespec *deadline;
	struct timespec *period;
        struct timespec *buffer1;
        unsigned long buffer2;
        unsigned long buffer3;
        struct mutex_data *mutex0;
        struct mutex_data *mutex1;
        int *n_freq;
        int n_frequency[20];
};

/*
struct rt_data {
	int tid;
	int prio;
	unsigned long exec_time;
        int util_taskset;
        unsigned long locked_usage;
        int mutex_flag;
        int high_freq;
	int max_util;
	struct timespec *deadline;
	struct timespec *period;
        struct timespec *buffer1;
        unsigned long buffer2;
        unsigned long buffer3;
        struct mutex_data *mutex0;
        struct mutex_data  mutex1;
        
         
};*/


typedef struct mutex_data chronos_mutex_t;

/* Set the basic information of the calling task */
long begin_rtseg_selfbasic(int prio, struct timespec* deadline,
			 struct timespec* period);

/* Set the basic information of another task */
long begin_rtseg_basic(int tid, int prio, struct timespec* deadline,
		     struct timespec* period);

/* Set the full rt information of the calling task */
long begin_rtseg_self(int prio, int max_util, int util_taskset,int mutex_flag, int high_freq,int* n_freq, chronos_mutex_t *m0, chronos_mutex_t *m1,unsigned long locked_usage, struct timespec* deadline,
		    struct timespec* period, unsigned long exec_time);

/* Set the full rt information of another task */
long begin_rtseg(int tid, int prio, int max_util, struct timespec* deadline,
		struct timespec* period, unsigned long exec_time);

/* End the real-time portion of the calling task */
long end_rtseg_self(int prio);

/* End the real-time portion of another task */
long end_rtseg(int tid, int prio);

/* Add an abort handler for the task */
long add_abort_handler_self(int max_util, struct timespec *deadline,
			 unsigned long exec_time);

/* Add an abort handler for another task */
long add_abort_handler(int tid, int max_util, struct timespec *deadline,
			 unsigned long exec_time);

/* Add an abort handler for the task with no deadline */
long add_abort_handler_selfnodeadline(int max_util, unsigned long exec_time);

/* Add an abort handler for another task with no deadline */
long add_abort_handler_nodeadline(int tid, int max_util, unsigned long exec_time);

long set_scheduler(int scheduler, int prio, unsigned long cpus);

/* Changed from class for ease in replacing pthread_mutext */
long chronos_mutex_init(chronos_mutex_t *m);
long chronos_mutex_destroy(chronos_mutex_t *m);
long chronos_mutex_lock(chronos_mutex_t *m);
long chronos_mutex_unlock(chronos_mutex_t *m);
int chronos_mutex_owner(chronos_mutex_t *m);

#ifdef __cplusplus
}
#endif

#endif

