#ifndef MCS_H
#define MCS_H

#include <linux/stddef.h>
#include <asm/atomic.h>

typedef	struct mcs_qnode * mcs_lock_t;
typedef struct mcs_qnode mcs_node_t;

struct mcs_qnode {
	struct mcs_qnode *next;
	char locked;
};

static inline void nop_pause(void)
{
	asm volatile("pause" : : : "memory");
}

static inline mcs_node_t * fetch_and_store(mcs_lock_t *ml, mcs_node_t *n)
{
	mcs_node_t *out = n;
#ifdef CONFIG_X86_32
	asm volatile("xchgl %k0, %1"
			: "=r" (out)
			: "m" (*ml), "0" (out)
			: "memory");
#else
	asm volatile("xchgq %0, %1"
			: "=r" (out)
			: "m" (*ml), "0" (out)
			: "memory");
#endif
	return out;
}

static inline mcs_node_t *cmp_and_swap(mcs_lock_t *ml, mcs_node_t *old, mcs_node_t *new)
{
	mcs_node_t *out;
#ifdef CONFIG_X86_32
	asm volatile("lock; cmpxchgl %k1, %2"
			: "=a" (out)
			: "r" (new), "m" (*ml), "0"(old)
			: "memory");
#else
	asm volatile("lock; cmpxchgq %1, %2"
			: "=a" (out)
			: "r" (new), "m" (*ml), "0"(old)
			: "memory");
#endif
	return out;
}

static inline void mcs_lock_init(mcs_lock_t *ml)
{
	*ml = NULL;
}

static inline void mcs_node_init(mcs_node_t *node)
{
	node->next = NULL;
	node->locked = 0;
}

static inline int mcs_is_locked(mcs_lock_t *ml)
{
	return *ml != NULL;
}

static inline int mcs_trylock(mcs_lock_t *ml, mcs_node_t *node)
{
	return cmp_and_swap(ml, NULL, node) == NULL;
}

static inline void mcs_lock(mcs_lock_t *ml, mcs_node_t *node)
{
	mcs_node_t *pred = fetch_and_store(ml, node);

	if(pred) {
		node->locked = 1;
		pred->next = node;
		while(node->locked)
			nop_pause();
	}
}

static inline void mcs_unlock(mcs_lock_t *ml, mcs_node_t *node)
{
	if(!node->next) {
		if(cmp_and_swap(ml, node, NULL) == node)
			return;

		while(!node->next)
			nop_pause();
	}

	node->next->locked = 0;
	node->next = NULL;
}
#endif
