/* chronos/gedf.c
 *
 * Global EDF Scheduler Module for ChronOS
 *
 * Author(s)
 *	- Matthew Dellinger, mdelling@vt.edu
 *
 * Copyright (C) 2009-2011 Virginia Tech Real Time Systems Lab
 */

#include <linux/module.h>
#include <linux/chronos_types.h>
#include <linux/chronos_sched.h>
#include <linux/list.h>

struct rt_info * sched_gedf(struct list_head *head, struct global_sched_domain *g)
{
	int count = 1, cpus = count_global_cpus(g);
	struct rt_info *it, *earliest = global_task(head->next);

	it = earliest;
	init_list(earliest, 0);

	list_for_each_entry_continue(it, head, task_list[1]) {
		count++;
		list_add_before(earliest, it, 0);

		if(count == cpus)
			break;
	}

	return earliest;
}

struct rt_sched_global gedf = {
	.base.name = "GEDF",
	.base.id = SCHED_RT_GEDF,
	.schedule = sched_gedf,
	.preschedule = presched_stw_generic,
	.arch = &rt_sched_arch_stw,
	.local = SCHED_RT_FIFO,
	.base.sort_key = SORT_KEY_DEADLINE,
	.base.list = LIST_HEAD_INIT(gedf.base.list)
};

static int __init gedf_init(void)
{
	return add_global_scheduler(&gedf);
}
module_init(gedf_init);

static void __exit gedf_exit(void)
{
	remove_global_scheduler(&gedf);
}
module_exit(gedf_exit);

MODULE_DESCRIPTION("Global EDF Scheduling Module for ChronOS");
MODULE_AUTHOR("Matthew Dellinger <matthew@mdelling.com>");
MODULE_LICENSE("GPL");

