/* gnpedf.c
 *
 * Global Non-Preemptible EDF Scheduler Module for ChronOS
 *
 * Author(s)
 *	- Matthew Dellinger, mdelling@vt.edu
 *
 * Copyright (C) 2009-2011 Virginia Tech Real Time Systems Lab
 */

#include <linux/module.h>
#include <linux/chronos_types.h>
#include <linux/chronos_sched.h>
#include <linux/list.h>
#include <linux/smp.h>

struct rt_info * sched_gnpedf(struct list_head *head, struct global_sched_domain *g)
{
	struct rt_info *task = global_task(head->next);
	_remove_task_global(task, g);
	return task;
}

struct rt_sched_global gnpedf = {
	.base.name = "G_NP_EDF",
	.base.id = SCHED_RT_GNPEDF,
	.schedule = sched_gnpedf,
	.preschedule = presched_concurrent_generic,
	.arch = &rt_sched_arch_concurrent,
	.local = SCHED_RT_FIFO,
	.base.sort_key = SORT_KEY_DEADLINE,
	.base.list = LIST_HEAD_INIT(gnpedf.base.list)
};

static int __init gnpedf_init(void)
{
	return add_global_scheduler(&gnpedf);
}
module_init(gnpedf_init);

static void __exit gnpedf_exit(void)
{
	remove_global_scheduler(&gnpedf);
}
module_exit(gnpedf_exit);

MODULE_DESCRIPTION("Global Non-Preemptible EDF Scheduling Module for ChronOS");
MODULE_AUTHOR("Matthew Dellinger <matthew@mdelling.com>");
MODULE_LICENSE("GPL");

