/* chronos/lbesa.c
 *
 * LBESA Single-Core Scheduler Module for ChronOS
 *
 * Author(s)
 *	- Matthew Dellinger, mdelling@vt.edu
 *
 * Copyright (C) 2009-2011 Virginia Tech Real Time Systems Lab
 */

#include <linux/module.h>
#include <linux/chronos_types.h>
#include <linux/chronos_sched.h>
#include <linux/list.h>

struct rt_info* sched_lbesa(struct list_head *head, int flags)
{
	struct rt_info *it, *best_ivd, *last_ivd, *best_dead;

	best_ivd = local_task(head->next);
	best_dead = best_ivd;
	initialize_lists(best_ivd);

	list_for_each_entry(it, head, task_list[0]) {
		if(check_task_failure(it, flags))
			return it;

		livd(it, false, flags);
		list_add_before(best_ivd, it, 0);
		list_add_before(best_dead, it, 1);

		if(it->local_ivd < best_ivd->local_ivd)
			best_ivd = it;
	}

	quicksort(best_ivd, 0, SORT_KEY_LVD, 0);
	last_ivd = best_ivd->list[0].prev;

	/* Remove the lowest VD task while the schedule is infeasible */
	do {
		if(list_is_feasible(best_dead, 1))
			break;

		if(last_ivd == best_dead)
			best_dead = best_dead->list[1].next;

		list_remove(last_ivd, 1);
		last_ivd = last_ivd->list[0].prev;
	} while (last_ivd != best_ivd);

	return best_dead;
}

struct rt_sched_local lbesa = {
	.base.name = "LBESA",
	.base.id = SCHED_RT_LBESA,
	.flags = 0,
	.schedule = sched_lbesa,
	.base.sort_key = SORT_KEY_DEADLINE,
	.base.list = LIST_HEAD_INIT(lbesa.base.list)
};

static int __init lbesa_init(void)
{
	return add_local_scheduler(&lbesa);
}
module_init(lbesa_init);

static void __exit lbesa_exit(void)
{
	remove_local_scheduler(&lbesa);
}
module_exit(lbesa_exit);

MODULE_DESCRIPTION("LBESA Scheduling Module for ChronOS");
MODULE_AUTHOR("Matthew Dellinger <matthew@mdelling.com>");
MODULE_LICENSE("GPL");

