/* chronos/rtdvs/base_edf.c
 *
 * Base-EDF Single-Core Scheduler Module for ChronOS
 *
 * Author(s)
 *	- Sonal Saha, sonal3@vt.edu
 *
 * Copyright (C) 2009-2011 Virginia Tech Real Time Systems Lab
 */

#include <linux/module.h>
#include <linux/chronos_types.h>
#include <linux/chronos_sched.h>
#include <linux/list.h>
#include <linux/cpufreq.h>
#include <asm/current.h>
//sonal3
struct rt_info* sched_base_edf(struct list_head *head, int flags)
{
         int cpu=0;
         struct cpufreq_policy *policy=NULL;

	struct rt_info *best = local_task(head->next);
        policy=per_cpu(cpufreq_cpu_data,cpu);

         __cpufreq_driver_target(policy, 2400000,CPUFREQ_RELATION_H);


	if(flags & SCHED_FLAG_PI)
		best = get_pi_task(best, head, flags);

	return best;
}

struct rt_sched_local base_edf = {
	.base.name = "BASE_EDF",
	.base.id = SCHED_RT_BASE_EDF,
	.flags = 0,
	.schedule = sched_base_edf,
	.base.sort_key = SORT_KEY_DEADLINE,
	.base.list = LIST_HEAD_INIT(base_edf.base.list)
};

static int __init base_edf_init(void)
{
	return add_local_scheduler(&base_edf);
}
module_init(base_edf_init);

static void __exit base_edf_exit(void)
{
	remove_local_scheduler(&base_edf);
}
module_exit(base_edf_exit);

MODULE_DESCRIPTION("BASE_EDF Single-Core Scheduling Module for ChronOS");
MODULE_AUTHOR("Sonal Saha <sonal3@vt.edu>");
MODULE_LICENSE("GPL");

