/* chronos/rtdvs/static_edf.c
 *
 * Base-EDF Single-Core Scheduler Module for ChronOS
 *
 * Author(s)
 *	- Sonal Saha, sonal3@vt.edu
 *
 * Copyright (C) 2009-2011 Virginia Tech Real Time Systems Lab
 */

#include <linux/module.h>
#include <linux/chronos_types.h>
#include <linux/chronos_sched.h>
#include <linux/list.h>
#include <linux/cpufreq.h>
#include <asm/current.h>

struct rt_info* sched_static_edf(struct list_head *head, int flags)
{
         int cpu=0;
         struct cpufreq_policy *policy=NULL;

	struct rt_info *best = local_task(head->next);
     policy=per_cpu(cpufreq_cpu_data,cpu);

     trace_printk(" best->util_taskset=%d\n", best->util_taskset);
        if((best->util_taskset)<= 49)
           __cpufreq_driver_target(policy, 1200000,CPUFREQ_RELATION_H);  
        if(((best->util_taskset)> 49) && ((best->util_taskset)<= 55) )
             __cpufreq_driver_target(policy, 1330000,CPUFREQ_RELATION_H);
        if(((best->util_taskset)> 55) && ((best->util_taskset)<= 61) )
             __cpufreq_driver_target(policy, 1470000,CPUFREQ_RELATION_H);
        if(((best->util_taskset)> 61) && ((best->util_taskset)<= 66) )
             __cpufreq_driver_target(policy, 1600000,CPUFREQ_RELATION_H);
        if(((best->util_taskset)> 66) && ((best->util_taskset)<= 72) )
             __cpufreq_driver_target(policy, 1730000,CPUFREQ_RELATION_L);
        if(((best->util_taskset)> 72) && ((best->util_taskset)<= 77) )
             __cpufreq_driver_target(policy, 1870000,CPUFREQ_RELATION_H);
        if(((best->util_taskset)> 77) && ((best->util_taskset)<= 83) )
             __cpufreq_driver_target(policy, 2000000,CPUFREQ_RELATION_H);
        if(((best->util_taskset)> 83) && ((best->util_taskset)<= 88) )
             __cpufreq_driver_target(policy, 2130000,CPUFREQ_RELATION_L);
        if(((best->util_taskset)> 88) && ((best->util_taskset)<= 94) )
             __cpufreq_driver_target(policy, 2270000,CPUFREQ_RELATION_H);
        if(((best->util_taskset)>94) )
             __cpufreq_driver_target(policy, 2400000,CPUFREQ_RELATION_H);


	if(flags & SCHED_FLAG_PI)
		best = get_pi_task(best, head, flags);

	return best;
}

struct rt_sched_local static_edf = {
	.base.name = "STATIC_EDF",
	.base.id = SCHED_RT_STATIC_EDF,
	.flags = 0,
	.schedule = sched_static_edf,
	.base.sort_key = SORT_KEY_DEADLINE,
	.base.list = LIST_HEAD_INIT(static_edf.base.list)
};

static int __init static_edf_init(void)
{
	return add_local_scheduler(&static_edf);
}
module_init(static_edf_init);

static void __exit static_edf_exit(void)
{
	remove_local_scheduler(&static_edf);
}
module_exit(static_edf_exit);

MODULE_DESCRIPTION("STATIC_EDF Single-Core Scheduling Module for ChronOS");
MODULE_AUTHOR("Sonal Saha <sonal3@vt.edu>");
MODULE_LICENSE("GPL");

